﻿using System;
using System.Diagnostics;
using System.Text;
using System.Windows;
using System.Windows.Media.Imaging;
using System.Windows.Threading;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace WPFTestInteractiveLayer
{
    public partial class Window1 : Window
    {
        private InMemoryMarkerOverlay markerOverlay;
        private RectangleShape worldExtent;

        public Window1()
        {
            InitializeComponent();
            worldExtent = new RectangleShape(-180, 90, 180, -90);
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            markerOverlay = new InMemoryMarkerOverlay();
            markerOverlay.FeatureSource.BeginTransaction();
            Random random = new Random();
            for (int i = 0; i < 500; i++)
            {
                double x = random.Next(-180, 180);
                double y = random.Next(-90, 90);
                PointShape point = new PointShape(x, y);
                markerOverlay.FeatureSource.AddFeature(point);
            }
            markerOverlay.FeatureSource.CommitTransaction();
            markerOverlay.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            Uri uriPath = new Uri("/Data1/Images/Truck.png", UriKind.RelativeOrAbsolute);
            markerOverlay.ZoomLevelSet.ZoomLevel01.DefaultPointMarkerStyle.ImageSource = new BitmapImage(uriPath);

            wpfMap1.Overlays.Add(markerOverlay);
            wpfMap1.CurrentExtent = worldExtent;
            wpfMap1.Refresh();
        }

        private void TestQuery_Click(object sender, RoutedEventArgs e)
        {
            TestQuery();
        }

        private void TestRendering_Click(object sender, RoutedEventArgs e)
        {
            TestRendering();
        }

        private void TestQuery()
        {
            StringBuilder builder = new StringBuilder();
            Stopwatch watch = new Stopwatch();
            for (int i = 0; i < 10; i++)
            {
                watch.Start();
                int markerCount = markerOverlay.GetMarkersToDraw(worldExtent).Count;
                builder.AppendLine(string.Format("Query {0} markers  Cost {1} milliseconds", markerCount, watch.ElapsedMilliseconds));
                watch.Reset();
            }
            MessageBox.Show(builder.ToString());
        }

        private void TestRendering()
        {
            foreach (var item in canvas.Children)
            {
                ((Marker)item).ImageSource = null;
            }
            canvas.Children.Clear();

            GeoCollection<Marker> markers = markerOverlay.GetMarkersToDraw(worldExtent);

            StringBuilder builder = new StringBuilder();

            Stopwatch watch = new Stopwatch();
            watch.Start();

            Random r = new Random();
            foreach (var marker in markers)
            {
                canvas.Children.Add(marker);
                marker.Draw(worldExtent, canvas.ActualWidth, canvas.ActualHeight);
            }

            builder.AppendLine(string.Format("Add {0} markers to canvas  Cost {1} milliseconds", canvas.Children.Count, watch.ElapsedMilliseconds));
            watch.Reset();
            watch.Start();

            canvas.Dispatcher.BeginInvoke(DispatcherPriority.Loaded,
                new Action(() =>
                {
                    builder.AppendLine(string.Format("Render {0} markers in canvas  Cost {1} milliseconds", canvas.Children.Count, watch.ElapsedMilliseconds));
                    MessageBox.Show(builder.ToString());
                }));
        }
    }
}
